-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftFrame")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Frame"] = true,
	["Frame options"] = true,
	["Width"] = true,
	["Sets the width of the frame"] = true,
	["Height"] = true,
	["Sets the height of the frame"] = true,

	["Background Color"] = true,
	["Sets the background color of the nameplate"] = true,
	["Border Color"] = true,
	["Sets the border color of the nameplate"] = true,
	["Border Style"] = true,
	["Sets the style of the nameplate border"] = true,
	
	["NONE"] = "None", 
	["STANDARD"] = "Standard", 
	["SQUARE"] = "Square",
} end)

-----------------------------------------------------------------------------

local borderData =
{
	["NONE"] 		= { texture = "", size = 0, inset = 0 },
	["STANDARD"] 	= { texture = "Interface\\Tooltips\\UI-Tooltip-Border", size = 16, inset = 5 },
	["SQUARE"]	 	= { texture = "Interface\\DialogFrame\\UI-DialogBox-Border", size = 16, inset = 5 },
}

-----------------------------------------------------------------------------

local AloftFrame = Aloft:NewModule("Frame", "AceEvent-2.0")
AloftFrame.db = Aloft:AcquireDBNamespace("frame")
AloftFrame.healthBarDB = Aloft:AcquireDBNamespace("healthBar")
Aloft:RegisterDefaults("frame", "profile", {
	borderStyle		= "NONE",
	borderColor		= { 1, 1, 1, 1 },
	backgroundColor	= { 0, 0, 0, 0 },
	width			= 10,
	height			= 10,
})

Aloft.Options.args.frame =
{
	type = 'group',
	name = L["Frame"],
	desc = L["Frame options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		width =
		{
			type = 'range',
			name = L["Width"],
			desc = L["Sets the width of the frame"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.width end,
			set = function(v) AloftFrame.db.profile.width = v AloftFrame:UpdateAll() end,
		},
		height =
		{
			type = 'range',
			name = L["Height"],
			desc = L["Sets the height of the frame"],
			min = 0,
			max = 100,
			step = 1,
			get = function() return AloftFrame.db.profile.height end,
			set = function(v) AloftFrame.db.profile.height = v AloftFrame:UpdateAll() end,
		},
		backgroundColor =
		{
			type = 'color',
			name = L["Background Color"],
			desc = L["Sets the background color of the nameplate"],
			get = function() return unpack(AloftFrame.db.profile.backgroundColor) end,
			set = function(r, g, b, a) AloftFrame.db.profile.backgroundColor = { r, g, b, a } AloftFrame:UpdateAll() end,
			hasAlpha = true,
		},
		borderColor =
		{
			type = 'color',
			name = L["Border Color"],
			desc = L["Sets the border color of the nameplate"],
			get = function() return unpack(AloftFrame.db.profile.borderColor) end,
			set = function(r, g, b, a) AloftFrame.db.profile.borderColor = { r, g, b, a } AloftFrame:UpdateAll() end,
			hasAlpha = true,
		},
		borderStyle =
		{
			type = 'text',
			name = L["Border Style"],
			desc = L["Sets the style of the nameplate border"],
			get = function() return AloftFrame.db.profile.borderStyle end,
			set = function(v) AloftFrame.db.profile.borderStyle = v AloftFrame:UpdateAll() end,
			validate = { ["NONE"] = L["NONE"], ["STANDARD"] = L["STANDARD"], ["SQUARE"] = L["SQUARE"] }
		},
	},
}

-----------------------------------------------------------------------------

function AloftFrame:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		self:SetupFrame(frame)
	end
	for frame in Aloft:IterateVisibleNameplates() do
		self:SetFrameSize(frame)
	end
end

-----------------------------------------------------------------------------

function AloftFrame:SetFrameSize(frame)
	-- This is a mechanism used to ensure that our bars are clickable...
	frame:SetHeight(self.db.profile.height)
	frame:SetWidth(self.db.profile.width)
end

function AloftFrame:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnNameplateShow", "SetFrameSize")

	self:UpdateAll()
end

function AloftFrame:OnDisable()
	self:UnregisterAllEvents()

	for frame in Aloft:IterateNameplates() do
		local backdropFrame = frame.aloftData.backdropFrame
		
		if backdropFrame then
			backdropFrame:Hide()
		end
		frame:SetFrameLevel(1)
	end
end

function AloftFrame:SetupFrame(frame)
	local profile = self.db.profile
	local aloftData = frame.aloftData

	if profile.borderStyle == "NONE" and profile.backgroundColor.a == 0 and not aloftData.backdropFrame then
		return
	end
	
	local backdropFrame = aloftData.backdropFrame
	if not backdropFrame then
		backdropFrame = CreateFrame("Frame", nil, frame)
		backdropFrame:SetPoint("CENTER", frame, "CENTER")
		
		aloftData.backdropFrame = backdropFrame
	end

	local data = borderData[profile.borderStyle]

	backdropFrame:SetBackdrop({
								bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
								edgeFile = data.texture, edgeSize = data.size,
								insets = {left = data.inset, right = data.inset, top = data.inset, bottom = data.inset},
							  })

	local healthBarProfile = self.healthBarDB.profile

	backdropFrame:SetBackdropColor(unpack(profile.backgroundColor))
	backdropFrame:SetBackdropBorderColor(unpack(profile.borderColor))
	backdropFrame:SetWidth(profile.width+healthBarProfile.width)
	backdropFrame:SetHeight(profile.height+healthBarProfile.height)
	backdropFrame:SetFrameLevel(1)
	frame:SetFrameLevel(2)
end

function AloftFrame:SetFrameSize(frame)
	local profile = self.db.profile
	local healthBarProfile = self.healthBarDB.profile

	frame:SetWidth(profile.width+healthBarProfile.width)
	frame:SetHeight(profile.height+healthBarProfile.height)
end

-----------------------------------------------------------------------------
